:- dynamic sol_fact/3, numsol/1.

display_solutions(lparse,NSol):-
	gettoken([0'\n],_,_), % remove first line
	set_count(numsol,0),
	show_next_solution_lparse,numsol(NSol).
	
display_solutions(dlv,NSol):-
	set_count(numsol,0),
	show_next_solution_dlv,numsol(NSol).

%%% lparse %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
show_next_solution_lparse:-
	gettoken([0'\n],T,_),!,
	(T='False',!
	;T='True',!
	;gettoken([0':],_,_), % remove text 'Stable Model:',
	 get0(_),             % remove blank
	 incr(numsol,1), numsol(N), writelist(['Answer:',N]),nl,
	 get_next_fact_lparse,
	 write_facts,nl,
	 show_next_solution_lparse
	).
show_next_solution_lparse.	

get_next_fact_lparse :-
	gettoken([0' ,0'\n],T,D),
	D=0' ,!,
	atom_to_term(T,Term,_),
	assert_sol_fact(Term),
	get_next_fact_lparse.
get_next_fact_lparse.	


%%% dlv %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

show_next_solution_dlv :-
	get0(C),C=0'{,!,    % new solution
	incr(numsol,1), numsol(N), writelist(['Answer:',N]),nl,
	get_next_fact_dlv,
	write_facts,nl,
	gettoken([0'\n],_,_), % remove up to breakline
	show_next_solution_dlv.
show_next_solution_dlv.	
	
get_next_fact_dlv :-
	gettoken([0' ,0'}],T,D),
	( D=0' ,!,
	 % remove last ',' from atom
	 atom_codes(T,Cs),append(Cs2,[0',],Cs),atom_codes(T2,Cs2),
	 atom_to_term(T2,Term,_),
	 assert_sol_fact(Term),
	 get_next_fact_dlv
	; % D=0'},
	 atom_to_term(T,Term,_),
	 assert_sol_fact(Term)
	).
	 
	
%%% common predicates %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

assert_sol_fact(Term):-
	(   Term =.. [F|_], functor_prefix(F,aux),!   % ignore auxiliary atoms
	  ; Term =.. [F|_], functor_prefix(F,known),!   % ignore auxiliary atoms
	  ; Term = -_X,!   % ignore auxiliary atoms	  
	  ; Term =.. [F|Args], length(Args,N), M is N-2, decl(func,F/M,_),!,
	    Args=[I|Args2],
	    assertz(sol_fact(I,F/M,Args2))
	  ; opt(debug),!,write(Term),nl
	  ; true
	).

gettoken(Delims,Tok,Delim):-
	gettokenchars(Delims,Chars,Delim),
	atom_codes(Tok,Chars).

gettokenchars(Delims,Cs,Delim):-
  get0(C),!,
  ( C = -1,!,fail
  ; member(C,Delims),!,Delim=C,Cs=[]  
  ; gettokenchars(Delims,Cs0,Delim), Cs=[C|Cs0]
  ).

write_facts:-
	findall((I - fact(F/N,Args)),sol_fact(I,F/N,Args),Facts),
	retractall(sol_fact(_,_,_)),
	keysort(Facts,Facts2),
	member(Fact,Facts2),write_fact(Fact),fail.
write_facts.	

functor_prefix(F,Pref):-
	atom_chars(F,FL),atom_chars(Pref,PL),
	append(PL,_,FL).
		
write_fact(I - fact(F/N,Args)):-
	append(Args0,[V],Args),
	decl(func,F/N,[Type,_,_,DV]),
	
	( DV=[V],\+ opt(debug), !  % don't show when default value and not debug
	; opt(only_actions), Type \= action,!
	;(Type=static,! % static attribute, don't show situation
	   ; writelist([I,': '])),
	
	  (V=false,!,write('not '); true),
	  write(F),
	  (Args0=[],!;write('('),writelist(',',Args0),write(')')),
	  ( V \= true, V \=false,!, writelist(['=',V]); true),nl
	).
